/*
** Module   : SequenceManager.java
** Abstract : Static abstract class that handles sequence functions and 
**             statements from Progress / OpenEdge 10.x. The actual 
**             implementation is delegated to one of the derived classes that
**             implements a strategy of handling the persistence of the 
**             sequences.
**
** Copyright (c) 2012, Golden Code Development Corporation.
** ALL RIGHTS RESERVED. Use is subject to license terms.
**
**           Golden Code Development Corporation
**                      CONFIDENTIAL
**
** -#- -I- --Date-- --JPRM-- ----------------Description-----------------
** 001 OM  20121116          Created initial version
*/

package com.goldencode.p2j.persist;

import java.util.*;
import com.goldencode.p2j.directory.*;
import com.goldencode.p2j.persist.sequence.SequenceHandler;
import com.goldencode.p2j.util.*;

/**
 * A class that abstracts all sequence handling functions and statements,
 * including static dynamic forms.
 * 
 * At the moment of initial commit we believe (and hope) that runtime
 * implementations of the statement/functions should leverage dialect
 * infrastructure in Hibernate if possible (if we find it is feasible to
 * use native sequences).
 * 
 * // db support for sequences should be tested with Dialect.supportsSequences()
 * 
 * The following schema properties (with the noted settings) must be 
 * supported by schema conversion:
 * <ul>
 *    <li>initial value of 0, 1 or 100
 *    <li>increment of 1
 *    <li>cycle on limit of no and yes
 *    <li>minimum value of 0, 1 or 100
 *    <li>maximum value (it is only sometimes explicitly specified) of 9999, 
 *          999999999 or 2000000000
 * </ul>
 */
public abstract class SequenceManager
{
   /**
    * Maps the <code>CURRENT-VALUE(seqName)</code> function from Progress as
    * it is defined in Open Edge 10.x.
    * Returns the current integer value of a sequence defined in the Data
    * Dictionary.
    *
    * @param   seqName
    *          An identifier that specifies the name of a sequence defined
    *          in the Data Dictionary.
    *
    * @return
    *       The current value of a sequence can be one of the following:
    *       <ul>
    *          <li>The initial value specified in the Data Dictionary.
    *          <li>The last value set with either the
    *             <code>CURRENT-VALUE</code> statement or the 
    *             <code>NEXT-VALUE</code> function.
    *          <li>The unknown value (<code>?</code>) if the sequence has 
    *             exceeded its minimum or maximum and is not cycling.
    *       </ul>
    */
   public static integer currentValue(String seqName)
   {
      // stub function
      return new integer();
   }

   /**
    * Maps the <code>CURRENT-VALUE(seqName, ldbName)</code> function from 
    * Progress as it is defined in Open Edge 10.x.
    * Returns the current integer value of the sequence defined in the Data
    * Dictionary.
    *
    * @param   seqName
    *          An identifier that specifies the name of a sequence defined
    *          in the Data Dictionary.
    * @param ldbName
    *          An identifier that specifies the logical name of the database
    *          in which the sequence is defined. The database must be
    *          connected. You can omit this parameter if the sequence name is
    *          unambiguous. If a sequence with this name exists in more than
    *          one connected database, then you must specify
    *          <code>ldbName</code>.
    *
    * @return
    *       The current value of a sequence can be one of the following:
    *       <ul>
    *          <li>The initial value specified in the Data Dictionary.
    *          <li>The last value set with either the
    *             <code>CURRENT-VALUE</code> statement or the 
    *             <code>NEXT-VALUE</code> function.
    *          <li>The unknown value (<code>?</code>) if the sequence has 
    *             exceeded its minimum or maximum and is not cycling.
    *       </ul>
    */
   public static integer currentValue(String seqName, String ldbName)
   {
      // stub function
      return new integer();
   }

   /**
    * Maps the <code>NEXT-VALUE(seqName)</code> function from Progress as it
    * is defined in Open Edge 10.x.
    * Returns the next integer value of the sequence, incremented by the
    * positive or negative value defined in the Data Dictionary.
    *
    * @param   seqName
    *          An identifier that specifies the name of a sequence defined
    *          in the Data Dictionary.
    *
    * @return
    *       The next value of a sequence can be one of the following:
    *       <ul>
    *          <li>The value obtained by incremented the current value of the 
    *             sequence with its increment if the value does not cross the
    *             upper/lower limits. In the case of cycling sequences the
    *             sequence value is warp around. 
    *          <li>The unknown value (<code>?</code>) if the sequence has 
    *             exceeded its minimum or maximum and is not cycling.
    *       </ul>
    */
   public static integer nextValue(String seqName)
   {
      // stub function
      return new integer();
   }

   /**
    * Maps the <code>NEXT-VALUE(seqName, ldbName)</code> function from
    * Progress as it is defined in Open Edge 10.x.
    * Returns the next integer value of the sequence, incremented by the
    * positive or negative value defined in the Data Dictionary.
    *
    * @param   seqName
    *          An identifier that specifies the name of a sequence defined
    *          in the Data Dictionary.
    * @param ldbName
    *          An identifier that specifies the logical name of the database
    *          in which the sequence is defined. The database must be
    *          connected. If multiple databases are connected, you can omit
    *          this parameter if you specify a sequence that is unique to
    *          one of the databases.
    *
    * @return
    *       The next value of a sequence can be one of the following:
    *       <ul>
    *          <li>The initial value specified in the Data Dictionary.
    *          <li>The last value set with either the
    *             <code>CURRENT-VALUE</code> statement or the 
    *             <code>NEXT-VALUE</code> function.
    *          <li>The unknown value (<code>?</code>) if the sequence has 
    *             exceeded its minimum or maximum and is not cycling.
    *       </ul>
    */
   public static integer nextValue(String seqName, String ldbName)
   {
      // stub function
      return new integer();
   }

   /**
    * Maps the <code>DYNAMIC-CURRENT-VALUE(seqName, ldbName)</code> function
    * from Progress as it is defined in Open Edge 10.x.
    * Returns the current integer value of the sequence defined in the Data
    * Dictionary.
    *
    * @param   seqName
    *          The name of a sequence defined in the Data Dictionary.
    * @param   ldbName
    *          The logical name of the database in which the sequence is 
    *          defined. The database must be connected.
    *
    * @return
    *       The current value of a sequence can be one of the following:
    *       <ul>
    *          <li>The initial value specified in the Data Dictionary.
    *          <li>The last value set with either the
    *             <code>CURRENT-VALUE</code> statement or the 
    *             <code>NEXT-VALUE</code> function.
    *          <li>The unknown value (<code>?</code>) if the sequence has 
    *             exceeded its minimum or maximum and is not cycling.
    *       </ul>
    */
   public static integer dynamicCurrentValue(String seqName, String ldbName)
   {
      // stub function
      return new integer();
   }

   /**
    * Maps the <code>DYNAMIC-CURRENT-VALUE(seqName, ldbName)</code> function
    * from Progress as it is defined in Open Edge 10.x.
    * Returns the current integer value of the sequence defined in the Data
    * Dictionary.
    *
    * @param   seqName
    *          The name of a sequence defined in the Data Dictionary.
    * @param   ldbName
    *          A character expression that evaluates to a logical name of 
    *          the database in which the sequence is defined. The database 
    *          must be connected.
    *
    * @return
    *       The current value of a sequence can be one of the following:
    *       <ul>
    *          <li>The initial value specified in the Data Dictionary.
    *          <li>The last value set with either the
    *             <code>CURRENT-VALUE</code> statement or the 
    *             <code>NEXT-VALUE</code> function.
    *          <li>The unknown value (<code>?</code>) if the sequence has 
    *             exceeded its minimum or maximum and is not cycling.
    *       </ul>
    */
   public static integer dynamicCurrentValue(String seqName,
                                             character ldbName)
   {
      // stub function
      return new integer();
   }

   /**
    * Maps the <code>DYNAMIC-CURRENT-VALUE(seqName, ldbName)</code> function
    * from Progress as it is defined in Open Edge 10.x.
    * Returns the current integer value of the sequence defined in the Data
    * Dictionary.
    *
    * @param   seqName
    *          A character expression that evaluates to the name of a
    *          sequence defined in the Data Dictionary.
    * @param   ldbName
    *          The logical name of the database in which the sequence is 
    *          defined. The database must be connected.
    *
    * @return
    *       The current value of a sequence can be one of the following:
    *       <ul>
    *          <li>The initial value specified in the Data Dictionary.
    *          <li>The last value set with either the
    *             <code>CURRENT-VALUE</code> statement or the 
    *             <code>NEXT-VALUE</code> function.
    *          <li>The unknown value (<code>?</code>) if the sequence has 
    *             exceeded its minimum or maximum and is not cycling.
    *       </ul>
    */
   public static integer dynamicCurrentValue(character seqName, 
                                             String ldbName)
   {
      // stub function
      return new integer();
   }

   /**
    * Maps the <code>DYNAMIC-CURRENT-VALUE(seqName, ldbName)</code> function
    * from Progress as it is defined in Open Edge 10.x.
    * Returns the current integer value of the sequence defined in the Data
    * Dictionary.
    *
    * @param   seqName
    *          A character expression that evaluates to the name of a
    *          sequence defined in the Data Dictionary.
    * @param ldbName
    *          A character expression that evaluates to a logical name of
    *          the database in which the sequence is defined. The database
    *          must be connected.
    *
    * @return
    *       The current value of a sequence can be one of the following:
    *       <ul>
    *          <li>The initial value specified in the Data Dictionary.
    *          <li>The last value set with either the
    *             <code>CURRENT-VALUE</code> statement or the 
    *             <code>NEXT-VALUE</code> function.
    *          <li>The unknown value (<code>?</code>) if the sequence has 
    *             exceeded its minimum or maximum and is not cycling.
    *       </ul>
    */
   public static integer dynamicCurrentValue(character seqName,
                                             character ldbName)
   {
      // stub function
      return new integer();
   }

   /**
    * Maps the <code>DYNAMIC-NEXT-VALUE(seqName, ldbName)</code> function
    * from Progress as it is defined in Open Edge 10.x.
    * Returns the next integer value of the sequence, incremented by the
    * positive or negative value defined in the Data Dictionary.
    *
    * @param   seqName
    *          The name of a sequence defined in the Data Dictionary.
    * @param   ldbName
    *          The logical name of the database in which the sequence is 
    *          defined. The database must be connected.
    *
    * @return
    *       The next value of a sequence can be one of the following:
    *       <ul>
    *          <li>The value obtained by incremented the current value of the 
    *             sequence with its increment if the value does not cross the
    *             upper/lower limits. In the case of cycling sequences the
    *             sequence value is warp around. 
    *          <li>The unknown value (<code>?</code>) if the sequence has 
    *             exceeded its minimum or maximum and is not cycling.
    *       </ul>
    */
   public static integer dynamicNextValue(String seqName,
                                          String ldbName)
   {
      // stub function
      return new integer();
   }

   /**
    * Maps the <code>DYNAMIC-NEXT-VALUE(seqName, ldbName)</code> function
    * from Progress as it is defined in Open Edge 10.x.
    * Returns the next integer value of the sequence, incremented by the
    * positive or negative value defined in the Data Dictionary.
    *
    * @param   seqName
    *          The name of a sequence defined in the Data Dictionary.
    * @param   ldbName
    *          A character expression that evaluates to a logical name of
    *          the database in which the sequence is defined. The database
    *          must be connected.
    *
    * @return
    *       The next value of a sequence can be one of the following:
    *       <ul>
    *          <li>The value obtained by incremented the current value of the 
    *             sequence with its increment if the value does not cross the
    *             upper/lower limits. In the case of cycling sequences the
    *             sequence value is warp around. 
    *          <li>The unknown value (<code>?</code>) if the sequence has 
    *             exceeded its minimum or maximum and is not cycling.
    *       </ul>
    */
   public static integer dynamicNextValue(String seqName,
                                          character ldbName)
   {
      // stub function
      return new integer();
   }

   /**
    * Maps the <code>DYNAMIC-NEXT-VALUE(seqName, ldbName)</code> function
    * from Progress as it is defined in Open Edge 10.x.
    * Returns the next integer value of the sequence, incremented by the
    * positive or negative value defined in the Data Dictionary.
    *
    * @param   seqName
    *          A character expression that evaluates to the name of a
    *          sequence defined in the Data Dictionary.
    * @param   ldbName
    *          The logical name of the database in which the sequence is 
    *          defined. The database must be connected.
    *
    * @return
    *       The next value of a sequence can be one of the following:
    *       <ul>
    *          <li>The value obtained by incremented the current value of the 
    *             sequence with its increment if the value does not cross the
    *             upper/lower limits. In the case of cycling sequences the
    *             sequence value is warp around. 
    *          <li>The unknown value (<code>?</code>) if the sequence has 
    *             exceeded its minimum or maximum and is not cycling.
    *       </ul>
    */
   public static integer dynamicNextValue(character seqName,
                                          String ldbName)
   {
      // stub function
      return new integer();
   }

   /**
    * Maps the <code>DYNAMIC-NEXT-VALUE(seqName, ldbName)</code> function
    * from Progress as it is defined in Open Edge 10.x.
    * Returns the next integer value of the sequence, incremented by the
    * positive or negative value defined in the Data Dictionary.
    *
    * @param   seqName
    *          A character expression that evaluates to the name of a
    *          sequence defined in the Data Dictionary.
    * @param ldbName
    *          A character expression that evaluates to a logical name of
    *          the database in which the sequence is defined. The database
    *          must be connected.
    *
    * @return
    *       The next value of a sequence can be one of the following:
    *       <ul>
    *          <li>The value obtained by incremented the current value of the 
    *             sequence with its increment if the value does not cross the
    *             upper/lower limits. In the case of cycling sequences the
    *             sequence value is warp around. 
    *          <li>The unknown value (<code>?</code>) if the sequence has 
    *             exceeded its minimum or maximum and is not cycling.
    *       </ul>
    */
   public static integer dynamicNextValue(character seqName,
                                          character ldbName)
   {
      // stub function
      return new integer();
   }

   /**
    * Maps the <code>CURRENT-VALUE(seqName) = expr</code> statement from
    * Progress as it is defined in Open Edge 10.x.
    * Resets the current integer value of the sequence defined in the Data
    * Dictionary.
    *
    * @param   seqName
    *          An identifier that specifies the name of a sequence defined
    *          in the Data Dictionary.
    * @param   newValue
    *          An integer expression assigned as the current value of the
    *          specified sequence. If expression is outside the boundary set
    *          by the initial value (at one end) and the lower limit or upper
    *          limit (at the other end) for the sequence, Progress returns an
    *          error, and the sequence value remains unchanged.
    */
   public static void setValue(String seqName, integer newValue)
   {
      // stub function
   }

   /**
    * Maps the <code>CURRENT-VALUE(seqName) = expr</code> statement from
    * Progress as it is defined in Open Edge 10.x.
    * Resets the current integer value of the sequence defined in the Data
    * Dictionary.
    *
    * @param   seqName
    *          An identifier that specifies the name of a sequence defined
    *          in the Data Dictionary.
    * @param   newValue
    *          An integer expression assigned as the current value of the
    *          specified sequence. If expression is outside the boundary set
    *          by the initial value (at one end) and the lower limit or upper
    *          limit (at the other end) for the sequence, Progress returns an
    *          error, and the sequence value remains unchanged.
    */
   public static void setValue(String seqName, int newValue)
   {
      // stub function
   }

   /**
    * Maps the <code>CURRENT-VALUE(seqName, ldbName) = expr</code> statement
    * from Progress as it is defined in Open Edge 10.x.
    * Resets the current integer value of the sequence defined in the Data
    * Dictionary.
    *
    * @param   seqName
    *          An identifier that specifies the name of a sequence defined
    *          in the Data Dictionary.
    * @param   ldbName
    *          An identifier that specifies the logical name of the database
    *          in which the sequence is defined. The database must be
    *          connected. You can omit this parameter if the sequence name is
    *          unambiguous. If more than one connected database has a sequence
    *          with given name, then you must supply <code>ldbName</code>.
    * @param   newValue
    *          An integer expression assigned as the current value of the
    *          specified sequence. If expression is outside the boundary set
    *          by the initial value (at one end) and the lower limit or upper
    *          limit (at the other end) for the sequence, Progress returns an
    *          error, and the sequence value remains unchanged.
    */
   public static void setValue(String seqName,
                               int newValue,
                               String ldbName)
   {
      // stub function
   }

   /**
    * Maps the <code>CURRENT-VALUE(seqName, ldbName) = expr</code> statement
    * from Progress as it is defined in Open Edge 10.x.
    * Resets the current integer value of the sequence defined in the Data
    * Dictionary.
    *
    * @param   seqName
    *          An identifier that specifies the name of a sequence defined
    *          in the Data Dictionary.
    * @param   ldbName
    *          An identifier that specifies the logical name of the database
    *          in which the sequence is defined. The database must be
    *          connected. You can omit this parameter if the sequence name is
    *          unambiguous. If more than one connected database has a sequence
    *          with given name, then you must supply <code>ldbName</code>.
    * @param   newValue
    *          An integer expression assigned as the current value of the
    *          specified sequence. If expression is outside the boundary set
    *          by the initial value (at one end) and the lower limit or upper
    *          limit (at the other end) for the sequence, Progress returns an
    *          error, and the sequence value remains unchanged.
    */
   public static void setValue(String seqName,
                               integer newValue,
                               String ldbName)
   {
      // stub function
   }

   /**
    * Maps the <code>DYNAMIC-CURRENT-VALUE(seqName, ldbName) = expr</code> 
    * statement from Progress as it is defined in Open Edge 10.x.
    * Resets the current integer value of the sequence defined in the Data
    * Dictionary.
    *
    * @param   seqName
    *          An identifier that specifies the name of a sequence defined
    *          in the Data Dictionary.
    * @param   ldbName
    *          An identifier that specifies the logical name of the database
    *          in which the sequence is defined. The database must be
    *          connected. You can omit this parameter if the sequence name is
    *          unambiguous. If more than one connected database has a sequence
    *          with given name, then you must supply <code>ldbName</code>.
    * @param   newValue
    *          An integer expression assigned as the current value of the
    *          specified sequence. If expression is outside the boundary set
    *          by the initial value (at one end) and the lower limit or upper
    *          limit (at the other end) for the sequence, Progress returns an
    *          error, and the sequence value remains unchanged.
    */
   public static void dynamicSetValue(character seqName,
                                      int newValue,
                                      character ldbName)
   {
      // stub function
   }

   /**
    * Maps the <code>DYNAMIC-CURRENT-VALUE(seqName, ldbName) = expr</code>
    * statement from Progress as it is defined in Open Edge 10.x.
    * Resets the current integer value of the sequence defined in the Data
    * Dictionary.
    *
    * @param   seqName
    *          A character expression that evaluates to the name of a 
    *          sequence defined in the Data Dictionary.
    * @param   ldbName
    *          A character expression that evaluates to the name of a database
    *          in which the sequence is defined. The database must be
    *          connected. You can omit this parameter if the sequence name is
    *          unambiguous. If more than one connected database has a sequence
    *          with given name, then you must supply <code>ldbName</code>.
    * @param   newValue
    *          An integer expression assigned as the current value of the
    *          specified sequence. If expression is outside the boundary set
    *          by the initial value (at one end) and the lower limit or upper
    *          limit (at the other end) for the sequence, Progress returns an
    *          error, and the sequence value remains unchanged.
    */
   public static void dynamicSetValue(character seqName,
                                      integer newValue,
                                      character ldbName)
   {
      // stub function
   }
}
